/** Performs a shallow update of an object using a partial object of the same shape. A new object is returned. */
export declare function update<Obj extends {}, K extends keyof Obj>(host: Obj, spec: Pick<Obj, K>): Obj;
/** Marker used to delete a key */
export declare const DELETE: undefined;
export declare type ObjectLiteral = object & {
    reduceRight?: 'nope';
};
export interface AtUpdater<TARGET, CURRENT> {
    __T: [TARGET, CURRENT];
    /**
     * Selects this Object key for update or further at() chaining
     */
    at<K extends keyof CURRENT>(this: AtUpdater<TARGET, ObjectLiteral>, key: K): Updater<TARGET, CURRENT[K]>;
    /**
     * Selects an Array index for update or further at() chaining
     */
    at<A>(this: AtUpdater<TARGET, A[]>, index: number): Updater<TARGET, A | undefined>;
}
export interface BoundAtUpdater<TARGET, CURRENT> {
    __T: [TARGET, CURRENT];
    /**
     * Selects this Object key for update or further at() chaining
     */
    at<K extends keyof CURRENT>(this: BoundAtUpdater<TARGET, ObjectLiteral>, key: K): BoundUpdater<TARGET, CURRENT[K]>;
    /**
     * Selects an Array index for update or further at() chaining
     */
    at<A>(this: BoundAtUpdater<TARGET, A[]>, index: number): BoundUpdater<TARGET, A | undefined>;
}
export interface Updater<TARGET, CURRENT> extends AtUpdater<TARGET, CURRENT> {
    __T: [TARGET, CURRENT];
    /**
     * Sets the value at the currently selected path.
     */
    set(value: CURRENT): (target: TARGET) => TARGET;
    /**
     * Modifies the value at the specified path. The current value is passed.
     */
    modify(modifier: (value: CURRENT) => CURRENT): (target: TARGET) => TARGET;
    /**
     * Makes the previous nullable chain level 'safe' by using a default value
     */
    withDefault<B, C extends B>(this: Updater<TARGET, B | undefined>, defaultValue: C): Updater<TARGET, B>;
    /**
     * Aborts the whole update operation if the previous chain level is null or undefined.
     */
    abortIfUndef<B>(this: Updater<TARGET, B | undefined>): Updater<TARGET, B>;
    /**
     * Aborts the whole update operation if the previous chain level doesn't verify a type guard
     */
    abortIfNot<C extends CURRENT>(predicate: (value: CURRENT) => value is C): Updater<TARGET, C>;
    /**
     * Aborts the whole update operation if the previous chain level doesn't verify a predicate
     */
    abortIfNot(predicate: (value: CURRENT) => boolean): Updater<TARGET, CURRENT>;
}
export interface BoundUpdater<TARGET, CURRENT> extends BoundAtUpdater<TARGET, CURRENT> {
    __T: [TARGET, CURRENT];
    /**
     * Sets the value at the currently selected path.
     */
    set(value: CURRENT): TARGET;
    /**
     * Modifies the value at the specified path. The current value is passed.
     */
    modify(modifier: (value: CURRENT) => CURRENT): TARGET;
    /**
     * Makes the previous nullable chain level 'safe' by using a default value
     */
    withDefault<B, C extends B>(this: BoundUpdater<TARGET, B | undefined>, defaultValue: C): BoundUpdater<TARGET, B>;
    /**
     * Aborts the whole update operation if the previous chain level is null or undefined.
     */
    abortIfUndef<B>(this: BoundUpdater<TARGET, B | undefined>): BoundUpdater<TARGET, B>;
    /**
     * Aborts the whole update operation if the previous chain level doesn't verify a type guard
     */
    abortIfNot<C extends CURRENT>(predicate: (value: CURRENT) => value is C): BoundUpdater<TARGET, C>;
    /**
     * Aborts the whole update operation if the previous chain level doesn't verify a predicate
     */
    abortIfNot(predicate: (value: CURRENT) => boolean): BoundUpdater<TARGET, CURRENT>;
}
export declare function deepUpdate<TARGET extends object>(target: TARGET): BoundAtUpdater<TARGET, TARGET>;
export declare function deepUpdate<TARGET extends object>(): AtUpdater<TARGET, TARGET>;
